/////////////////////////////////////////////////////////////////////////
//
// File: texgen.cpp : Defines the entry point for the console application.
//
//
// SDK Version: 8
//
// Desc: Texture generator.
//
//
// Parameters: 
//		1st - the name of the height map (must be 24bit bmp file)
//		2nd - number of mipmaps (use 1)
//
// Copyright (c) 2001 Yordan Gyurchev
//------------------------------------------------------------------------

#include "warjo_tertex.h"
#include "warjo_pscript.h"

//////////////////////////////////////////////////////////////////////////
//
//the entry point for the console application
//
int main(int argc, char* argv[])
{
	int initRes=128;

	//load the height map
	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmch;
	FILE *fp=fopen(argv[1],"rb");
	if (fp==NULL) return 0;

	//read headers
	fread(&bmfh,sizeof(bmfh),1,fp);
	fread(&bmch,sizeof(bmch),1,fp);

	// load the height map
	BYTE *pbmap=(BYTE *)malloc(bmch.biWidth*bmch.biHeight);
	for(int i=0;i<bmch.biWidth*bmch.biHeight;i++) {
		pbmap[i]=fgetc(fp);
		fgetc(fp);fgetc(fp);
	}
	fclose(fp);
	
	//get the mips number
	int mips=atoi(argv[2]);
	if (mips<1) mips=1;
	if (mips>4) mips=4;

	//printf height map info
	printf(" Name: %s\n\n Size:%dx%d\n\n\n",argv[1],bmch.biWidth,bmch.biHeight);
	
	printf("Loading terrains.txt ...\n\n");

	//the resource singleton
	wj_resource res;

	//the script singleton
	PureScript ps;

	//map all static values from the wj_texture_gen
	wj_texture_gen::Init();
	
	//map the inital resulution
	ps.Map(&initRes,"InitialResolution");

	//load the txt file
	ps.Load("terrains.txt");


	int iResolution=initRes;

	//iterate through resolutions ((for all mipmaps)

	for(;mips>0;mips--,iResolution/=2) {

		//new texture object for generation
		wj_texture_gen tg(iResolution,iResolution);
	
		//for every patch in this heght map
		for(int yo=0;yo<bmch.biHeight;yo+=wj_texture_gen::pside)
			for(int xo=0;xo<bmch.biWidth;xo+=wj_texture_gen::pside)
			{
			//show feedback
			printf("Generating %dx%d texture...",iResolution,iResolution);

			//make the texture
			tg.MakeTexture(pbmap,bmch.biWidth,bmch.biHeight,xo,yo);

			//get patch
			int px=xo/wj_texture_gen::pside;
			int py=yo/wj_texture_gen::pside;

			//mirror the texture alterantively with the changing of the patch
			tg.Mirror(px&1,py&1);
	
			//save the texture...
			char szName[]="texture";
			char szfName[128];

			//construct the name
			sprintf(szfName,"%s%dat%dx%d.bmp",szName,iResolution,px,py);
	
			//the header (reuse the file header)
			BITMAPINFOHEADER bmchn;
			memmove(&bmchn,&bmch,sizeof(bmchn));

			//adjust te header information
			bmchn.biHeight=bmchn.biWidth=iResolution;
			bmfh.bfSize=sizeof(bmfh)+sizeof(bmchn)+(iResolution*iResolution)*3;
			
			//write the file
			fp=fopen(szfName,"wb");
			if (fp==NULL) return 0;
			
			//headers
			fwrite(&bmfh,1,sizeof(bmfh),fp);
			fwrite(&bmchn,1,sizeof(bmchn),fp);
			
			//data
			fwrite(tg.bitmap,3,iResolution*iResolution,fp);
			fclose(fp);
			printf("written!\n");
			}
		}

	return 0;
}
